# Poisson-Tweedie: Properties and regression models for count data -----
# Author: Wagner Hugo Bonat LEG/IMADA ----------------------------------
# Script 3 - Underdispersed data ---------------------------------------
# Date: 11/07/2016 -----------------------------------------------------
rm(list=ls())

# Loading extra packages
require(mcglm)
require(Matrix)
require(lattice)
require(latticeExtra)

# Loading extra functions
source("PTWfunctions.R")

# Loading data set
est.ing <- c("vegetative","flower bud","blossom","fig","cotton boll")
cap <- expand.grid(rept=1:5,
                   des=seq(0,100,l=5)/100,
                   est=factor(est.ing, levels=est.ing))
cap$nc <- c(10, 9, 8, 8, 10, 11, 9, 10, 10, 10, 8, 8, 10, 8, 9, 9, 7, 7,
            8, 9, 8, 6, 6, 5, 6, 7, 8, 8, 9, 10, 9, 12, 7, 10, 9, 8, 9,
            9, 10, 8, 11, 10, 7, 8, 8, 7, 7, 7, 7, 8, 10, 9, 8, 12, 8,
            7, 5, 5, 7, 5, 6, 5, 7, 4, 7, 8, 5, 7, 6, 4, 5, 5, 4, 4, 5,
            8, 10, 7, 8, 10, 9, 6, 6, 8, 6, 9, 7, 11, 8, 9,6, 6, 6, 6,
            7, 3, 3, 2, 4, 3, 11, 7, 9, 12 , 11, 9, 13, 8, 10, 10, 9, 7,
            7, 9, 9, 8, 8, 10, 8, 10, 9, 8, 10, 8, 10)

# Linear predictor
form <- nc ~ est:(des+I(des^2))

# Fitting Poisson model
fit0 <- glm(form, data = cap, family = poisson)

# Fitting Poisson-Tweedie model
fit1 <- mcglm(c(form), list(mc_id(cap)), link = "log", 
              variance = "poisson_tweedie", power_fixed = FALSE, 
              data = cap,
              control_algorithm = list(verbose = FALSE, max_iter = 100, 
                                       tunning = 0.5, correct = FALSE))
summary(fit1)

# Table 3
cbind("Poisson-Tweedie" = coef(fit1, type = "beta")$Estimates, 
      "Poisson" = coef(fit0),
      "Ratio" = coef(fit1, type = "beta")$Estimates/coef(fit0))

# Standard errors
cbind("Poisson-Tweedie" = coef(fit1, type = "beta", std.error = TRUE)$Std.error, 
      "Poisson" = sqrt(diag(vcov(fit0))),
      "Ratio" = coef(fit1, type = "beta", std.error = TRUE)$Std.error/sqrt(diag(vcov(fit0))))

# Figure 7 
des <- seq(0, 1, l = 100)
est <- unique(cap$est)
data.new <- expand.grid(des,est)
names(data.new) <- c("des","est")
X.new <- model.matrix( ~ est:(des + I(des^2)), data = data.new)
eta <- X.new%*%coef(fit1, type = "beta")$Estimates
std.error <- sqrt(diag(as.matrix(X.new%*%vcov(fit1)[1:11,1:11]%*%t(X.new))))
data.new$pred <- exp(eta)
data.new$lim.min <- exp(eta - qnorm(0.975)*std.error)
data.new$lim.max <- exp(eta + qnorm(0.975)*std.error)

p1 <- xyplot(nc~des|est, data=cap, layout=c(5,1), 
             xlim=extendrange(c(0:1),f=0.15),
             xlab="Artificial defoliation level",
             ylab="Number of bolls produced", type=c("p"),
             col.line="gray50",
             panel=panel.beeswarm,
             par.settings = ps,
             strip=strip.custom(bg="gray90", factor.levels=est.ing),
             r=0.05) +
    as.layer(xyplot(pred ~ des|est, data=data.new, 
                    col.line="black", type = "l", lty = 2,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing))) +
    as.layer(xyplot(lim.min ~ des|est, data=data.new, 
                    col.line="black", type = "l", lty = 1,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing))) +
    as.layer(xyplot(lim.max ~ des|est, data=data.new, 
                    col.line="black", type = "l", lty = 1,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing)))
print(p1)

# Poisson fitted values
X.new <- model.matrix( ~ est:(des + I(des^2)), data = data.new)
eta.poisson <- X.new%*%coef(fit0)
std.error.poisson <- sqrt(diag(as.matrix(X.new%*%vcov(fit0)%*%t(X.new))))
data.new$pred.poisson <- exp(eta.poisson)
data.new$lim.min.poisson <- exp(eta.poisson - qnorm(0.975)*std.error.poisson)
data.new$lim.max.poisson <- exp(eta.poisson + qnorm(0.975)*std.error.poisson)

p2 <- xyplot(nc~des|est, data=cap, layout=c(5,1), 
             xlim=extendrange(c(0:1),f=0.15),
             xlab="Artificial defoliation level",
             ylab="Number of bolls produced", type=c("p"),
             col.line="gray50",
             panel=panel.beeswarm,
             par.settings = ps,
             strip=strip.custom(bg="gray90", factor.levels=est.ing),
             r=0.05) +
    as.layer(xyplot(pred ~ des|est, data=data.new, 
                    col.line="black", type = "l", lty = 2, lwd = 2,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing))) +
    as.layer(xyplot(lim.min ~ des|est, data=data.new, 
                    col.line="black", type = "l", lty = 1,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing))) +
    as.layer(xyplot(lim.max ~ des|est, data=data.new, 
                    col.line="black", type = "l", lty = 1,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing))) +
    as.layer(xyplot(pred.poisson ~ des|est, data=data.new, 
                     col.line="gray", type = "l", lty = 3, lwd = 2,
                     strip=strip.custom(bg="gray90", factor.levels=est.ing))) +
    as.layer(xyplot(lim.max.poisson ~ des|est, data=data.new, 
                    col.line="gray", type = "l", lty = 1,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing))) +
    as.layer(xyplot(lim.min.poisson ~ des|est, data=data.new, 
                    col.line="gray", type = "l", lty = 1,
                    strip=strip.custom(bg="gray90", factor.levels=est.ing)))

print(p2)

# END ------------------------------------------------------------------

