# Poisson-Tweedie: properties and regression models for count data -----
# Author: Wagner Hugo Bonat LEG/IMADA ----------------------------------
# Script 4 - Ionizing radiation: Zero-inflated  ------------------------
# Date: 11/07/2016 -----------------------------------------------------
rm(list=ls())

# Loading extra packages
require(mcglm)
require(Matrix)
require(statmod)
require(tweedie)
require(bbmle)

# Loading extra functions
source("PTWfunctions.R")
source("PTWfitcountdist.R")

# Loading data set -----------------------------------------------------
# Data set B.1
dose<-c(0.1,0.3,0.5,0.7,1)
freq<-matrix(c(2281,847,567,356,169,130,127,165,167,131,21,19,49,62,72,
               1,6,16,9,18,0,1,2,5,9,0,0,0,1,0,0,0,0,0,0,0,0,0,0,1),5,8)
d<-dim(freq)[1]
ndic<-dim(freq)[2]-1
ncel<-rowSums(freq)
X<-as.vector(freq%*%(0:ndic))
dic<-dosevec<-numeric()
for(i in 1:d){
    for (j in 0:ndic){
        dic<-c(dic,rep(j,freq[i,j+1]))
        dosevec<-c(dosevec,rep(dose[i],freq[i,j+1]))
    }}
datavecA1<-data.frame(dic,dosevec,dosevec2=dosevec^2) 
dataggrA1<-data.frame(X,ncel,dose,dose2=dose^2)
datavec<- datavecA1; dataggr<-dataggrA1

# Linear predictor
form <- dic ~ dosevec + dosevec2

# Poisson model --------------------------------------------------------
fit0 <- glm(form, family = "poisson", data = datavec)

# Poisson-Tweedie model ------------------------------------------------
fit1 <- mcglm(c(form), list(mc_id(datavec)), link = "log", 
              variance = "poisson_tweedie", power_fixed = FALSE, 
              data = datavec,
              control_algorithm = list(verbose = FALSE, max_iter = 1000,
                                       tunning = 0.5, tol = 1e-10))

# Estimates
round(cbind(coef(fit0),coef(fit1, type = "beta")$Estimates),3)
round(cbind(sqrt(diag(vcov(fit0))), 
      coef(fit1, type = "beta", std.error = TRUE)$Std.error),3)

round(cbind(coef(fit1, type = "beta")$Estimates/coef(fit0),
    coef(fit1, type = "beta", std.error = TRUE)$Std.error/sqrt(diag(vcov(fit0)))),3)

# Zero-inflated negative binomial model --------------------------------
# Starting values for the zero-part for zero-inflated models
# Loading function from 
dic0<-as.integer(datavec$dic<=0)
start0c <- glm(dic0~dosevec, family=binomial(link = "logit"), 
               data=datavec)$coefficients
mustart <- coef(fit0)
ZINB1clog <- fitcountdist(dic~dosevec + dosevec2|1+dosevec, data=datavec, 
                          dist="ZINB1", link="log", 
                          start=c(mustart,start0c,0.1))

ZIPc <- fitcountdist(dic~dosevec+dosevec2|dosevec, data=datavec, dist="ZIP", 
                   link="log", start=c(mustart,start0c))

# Computing the log-likelihood for Poisson-Tweedie model ---------------
data.new <- data.frame("dosevec" = unique(datavec$dosevec), 
                       "dosevec2" = unique(datavec$dosevec2))
X <- model.matrix(~ dosevec + dosevec2, data = data.new)
eta = X%*%coef(fit1, type = "beta")$Estimates
std.error <- sqrt(diag(as.matrix(X%*%vcov(fit1)[1:3,1:3]%*%t(X))))

# Fitted values and confidence interval
pred <- exp(eta)
lim.min <- exp(eta - qnorm(0.975)*std.error)
lim.max <- exp(eta + qnorm(0.975)*std.error)

# Observed values
x.levels <- data.new$dosevec
obs <- as.numeric(tapply(datavec$dic, datavec$dosevec, mean))
plot(obs ~ x.levels, ylim = c(0,1.5))
lines(x.levels, pred)
lines(x.levels, lim.min)
lines(x.levels, lim.max)

# Fitted distribution
phi.ptw <- coef(fit1, type = "tau")$Estimates
power.ptw <- coef(fit1, type = "power")$Estimates
pred.mat.ptw <- matrix(NA, ncol = 5, nrow = 7)
for(i in 1:5) {
    temp <- dptweedie(c(0:5,7), mu = pred[i], phi = phi.ptw, power = power.ptw, 
                      n_pts = 180, method = "laguerre")
    pred.mat.ptw[,i] <- temp
}

# Observed data --------------------------------------------------------
obs.dist <- table(dic, dosevec)

# Log-likelihood value - Poisson-Tweedie model -------------------------
sum(log(pred.mat.ptw)*obs.dist)

# Log-Likelihood value - Zero-Inflated negative binomial model ---------
ZINB1clog$loglik
ZIPc$loglik

# Log-likelihood value - Poisson model ---------------------------------
logLik(fit0)

# END ------------------------------------------------------------------

